using System;
using System.Collections;

namespace AutomataPattern
{
	public class TransitionContainer
	{
		private ArrayList _container;

		private class Transition
		{
			private State _stateFrom;
			private State _stateTo;
			private Condition _condition;
			private ArrayList _actionList;

			public ArrayList ActionList
			{
				get { return _actionList; }
				set { _actionList = value; }
			}

			public State StateFrom
			{	
				get { return _stateFrom; }
				set { _stateFrom = value; }
			}
		
			public State StateTo
			{	
				get { return _stateTo; }
				set { _stateTo = value; }
			}

			public Condition Condition
			{	
				get { return _condition; }
				set { _condition = value; }
			}

			public Transition( State stateFrom, State stateTo, Condition condition, ArrayList actionList )
			{
				_stateFrom = stateFrom;
				_stateTo = stateTo;
				_condition = condition;
				_actionList = actionList;
			}
		}

		public TransitionContainer()
		{
			_container = new ArrayList();
		}

		public void Add( State stateFrom, State stateTo, Condition condition, Action action )
		{
			ArrayList actionList = new ArrayList();
			actionList.Add( action );
			Add(stateFrom, stateTo, condition, actionList);
		}

		public void Add( State stateFrom, State stateTo, Condition condition, ArrayList actionList )
		{
			foreach ( Transition transition in _container )
				if ( (transition.StateFrom == stateFrom) && (transition.StateTo == stateTo) && (transition.Condition == condition) )
					throw new ApplicationException("Transition already exists");

			_container.Add(new Transition(stateFrom, stateTo, condition, actionList) );			
		}

		public State GetNextState( State state, int e )
		{
			Transition _default = null;

			foreach (Transition transition in _container)
				if (transition.StateFrom == state)
				{
					if (transition.Condition == null)
						_default = transition;
					else if (transition.Condition.Test( e ))
					{
						if (transition.ActionList != null)
						{
							foreach ( Action a in transition.ActionList )
							{
								if ( a != null)
									a.Do();
							}
						}
						if (transition.StateFrom != transition.StateTo)
							transition.StateTo.ExecuteIn();
						return transition.StateTo;
					}
				}
		
			if (_default != null)
			{
				if (_default.ActionList != null)
				{
					foreach ( Action a in _default.ActionList )
					{
						a.Do();
					}
				}
				if (_default.StateFrom != _default.StateTo)
					_default.StateTo.ExecuteIn();
				return _default.StateTo;
			}
			return state;
		}
	}
}
