using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;
using System.IO;
using AutomataPattern;



namespace TrafficLights
{
	//  
	public class MainForm : System.Windows.Forms.Form
	{
		private Graphics _graphics;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Timer blinkTimer;
		public System.Windows.Forms.Timer longTimer;
		public System.Windows.Forms.Timer shortTimer;
		protected Automata0 a0;
		protected Automata0.states currState;
		private System.Windows.Forms.Button greenFixedButton;
		private System.Windows.Forms.Button yellowFixedButton;
		private System.Windows.Forms.Button redFixedButton;
		private System.Windows.Forms.Button greenBrokenButton;
		private System.Windows.Forms.Button yellowBrokenButton;
		private System.Windows.Forms.Button redBrokenButton;
		private System.Windows.Forms.Label fixedLabel;
		private System.Windows.Forms.Label brokenLabel;
		private System.Windows.Forms.GroupBox hooligansGroupBox;
		private System.Windows.Forms.Label offLabel;
		private System.Windows.Forms.Label onLabel;
		private System.Windows.Forms.Label autoLabel;
		private System.Windows.Forms.Label manualLabel;
		private System.Windows.Forms.Label blinkLabel;
		private System.Windows.Forms.TrackBar smallButtonSlider;
		private System.Windows.Forms.TrackBar modeSlider;
		private System.Windows.Forms.Button switchButton;
		private System.Windows.Forms.Button powerButton;
		private System.Windows.Forms.GroupBox controlPanelGroupBox;
		private System.Windows.Forms.Button smallButton;
		private System.Windows.Forms.Panel drawPanel;
		private bool _showLight;

		//   
		public void DrawElement( int index, Brush brush )
		{
			int d = (drawPanel.Width > drawPanel.Height) ? drawPanel.Height / 3 : drawPanel.Width / 3;
			_graphics.FillEllipse( brush, drawPanel.Width / 2 - d / 2, (drawPanel.Height - d*3)/2 + d*index, d, d);
		}

		//  
		public void Redraw(Automata0.states what)
		{
			_graphics.FillRectangle( Brushes.AliceBlue, 0, 0, drawPanel.Width, drawPanel.Height);			

			for (int i = 0; i < 3; i++)
				DrawElement(i, Brushes.Black);

			switch(what)
			{
				case Automata0.states.GREEN:
					_showLight = false;
					DrawElement(2, Brushes.Green);
					break;
				case Automata0.states.BLINKGREEN:
					if (_showLight)
						DrawElement(2, Brushes.Green);
					else
						DrawElement(2, Brushes.Black);
					blinkTimer.Start();
					break;
				case Automata0.states.BLINKYELLOW:
					if (_showLight)
						DrawElement(1, Brushes.Yellow);
					else
						DrawElement(1, Brushes.Black);
					blinkTimer.Start();
					break;
				case Automata0.states.YELLOW:
					_showLight = false;
					DrawElement(1, Brushes.Yellow);
					break;
				case Automata0.states.RED:
					_showLight = false;
					DrawElement(0, Brushes.Red);
					break;
				case Automata0.states.REDYELLOW:
					_showLight = false;
					DrawElement(0, Brushes.Red);
					DrawElement(1, Brushes.Yellow);
					break;
				case Automata0.states.OFF:
					_showLight = false;
					break;

			}
		}

		//   
		public void ChangeLight(Automata0.states what)
		{
			currState = what;
			Redraw( what );
		}

		//  
		public MainForm()
		{
			//  			
			StreamWriter sw = new StreamWriter("test.log", false, System.Text.Encoding.Default);

			sw.AutoFlush = true;
			
			TextWriterTraceListener myWriter = new 
				TextWriterTraceListener( sw );
			Trace.Listeners.Add(myWriter);

			//    
			currState = Automata0.states.OFF;

			InitializeComponent();

			_graphics = drawPanel.CreateGraphics();
			powerButton.BackColor = Color.Red;

			//  
			a0 = new Automata0( null, this );
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.drawPanel = new System.Windows.Forms.Panel();
			this.greenFixedButton = new System.Windows.Forms.Button();
			this.yellowFixedButton = new System.Windows.Forms.Button();
			this.redFixedButton = new System.Windows.Forms.Button();
			this.greenBrokenButton = new System.Windows.Forms.Button();
			this.yellowBrokenButton = new System.Windows.Forms.Button();
			this.redBrokenButton = new System.Windows.Forms.Button();
			this.fixedLabel = new System.Windows.Forms.Label();
			this.brokenLabel = new System.Windows.Forms.Label();
			this.hooligansGroupBox = new System.Windows.Forms.GroupBox();
			this.offLabel = new System.Windows.Forms.Label();
			this.onLabel = new System.Windows.Forms.Label();
			this.autoLabel = new System.Windows.Forms.Label();
			this.manualLabel = new System.Windows.Forms.Label();
			this.blinkLabel = new System.Windows.Forms.Label();
			this.smallButtonSlider = new System.Windows.Forms.TrackBar();
			this.modeSlider = new System.Windows.Forms.TrackBar();
			this.switchButton = new System.Windows.Forms.Button();
			this.powerButton = new System.Windows.Forms.Button();
			this.controlPanelGroupBox = new System.Windows.Forms.GroupBox();
			this.smallButton = new System.Windows.Forms.Button();
			this.blinkTimer = new System.Windows.Forms.Timer(this.components);
			this.longTimer = new System.Windows.Forms.Timer(this.components);
			this.shortTimer = new System.Windows.Forms.Timer(this.components);
			this.hooligansGroupBox.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.smallButtonSlider)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.modeSlider)).BeginInit();
			this.controlPanelGroupBox.SuspendLayout();
			this.SuspendLayout();
			// 
			// drawPanel
			// 
			this.drawPanel.Location = new System.Drawing.Point(16, 16);
			this.drawPanel.Name = "drawPanel";
			this.drawPanel.Size = new System.Drawing.Size(168, 272);
			this.drawPanel.TabIndex = 0;
			// 
			// greenFixedButton
			// 
			this.greenFixedButton.Location = new System.Drawing.Point(232, 88);
			this.greenFixedButton.Name = "greenFixedButton";
			this.greenFixedButton.Size = new System.Drawing.Size(64, 23);
			this.greenFixedButton.TabIndex = 7;
			this.greenFixedButton.Text = "";
			this.greenFixedButton.Click += new System.EventHandler(this.button9_Click);
			// 
			// yellowFixedButton
			// 
			this.yellowFixedButton.Location = new System.Drawing.Point(232, 56);
			this.yellowFixedButton.Name = "yellowFixedButton";
			this.yellowFixedButton.Size = new System.Drawing.Size(64, 23);
			this.yellowFixedButton.TabIndex = 6;
			this.yellowFixedButton.Text = "";
			this.yellowFixedButton.Click += new System.EventHandler(this.button8_Click);
			// 
			// redFixedButton
			// 
			this.redFixedButton.Location = new System.Drawing.Point(232, 24);
			this.redFixedButton.Name = "redFixedButton";
			this.redFixedButton.Size = new System.Drawing.Size(64, 23);
			this.redFixedButton.TabIndex = 5;
			this.redFixedButton.Text = "";
			this.redFixedButton.Click += new System.EventHandler(this.button7_Click);
			// 
			// greenBrokenButton
			// 
			this.greenBrokenButton.Location = new System.Drawing.Point(88, 88);
			this.greenBrokenButton.Name = "greenBrokenButton";
			this.greenBrokenButton.Size = new System.Drawing.Size(64, 23);
			this.greenBrokenButton.TabIndex = 4;
			this.greenBrokenButton.Text = "";
			this.greenBrokenButton.Click += new System.EventHandler(this.button6_Click);
			// 
			// yellowBrokenButton
			// 
			this.yellowBrokenButton.Location = new System.Drawing.Point(88, 56);
			this.yellowBrokenButton.Name = "yellowBrokenButton";
			this.yellowBrokenButton.Size = new System.Drawing.Size(64, 23);
			this.yellowBrokenButton.TabIndex = 3;
			this.yellowBrokenButton.Text = "";
			this.yellowBrokenButton.Click += new System.EventHandler(this.button5_Click);
			// 
			// redBrokenButton
			// 
			this.redBrokenButton.Location = new System.Drawing.Point(88, 24);
			this.redBrokenButton.Name = "redBrokenButton";
			this.redBrokenButton.Size = new System.Drawing.Size(64, 23);
			this.redBrokenButton.TabIndex = 2;
			this.redBrokenButton.Text = "";
			this.redBrokenButton.Click += new System.EventHandler(this.button4_Click);
			// 
			// fixedLabel
			// 
			this.fixedLabel.Location = new System.Drawing.Point(168, 56);
			this.fixedLabel.Name = "fixedLabel";
			this.fixedLabel.Size = new System.Drawing.Size(56, 23);
			this.fixedLabel.TabIndex = 1;
			this.fixedLabel.Text = "";
			// 
			// brokenLabel
			// 
			this.brokenLabel.Location = new System.Drawing.Point(8, 56);
			this.brokenLabel.Name = "brokenLabel";
			this.brokenLabel.Size = new System.Drawing.Size(56, 23);
			this.brokenLabel.TabIndex = 0;
			this.brokenLabel.Text = "";
			// 
			// hooligansGroupBox
			// 
			this.hooligansGroupBox.Controls.Add(this.greenFixedButton);
			this.hooligansGroupBox.Controls.Add(this.yellowFixedButton);
			this.hooligansGroupBox.Controls.Add(this.redFixedButton);
			this.hooligansGroupBox.Controls.Add(this.greenBrokenButton);
			this.hooligansGroupBox.Controls.Add(this.yellowBrokenButton);
			this.hooligansGroupBox.Controls.Add(this.redBrokenButton);
			this.hooligansGroupBox.Controls.Add(this.fixedLabel);
			this.hooligansGroupBox.Controls.Add(this.brokenLabel);
			this.hooligansGroupBox.Location = new System.Drawing.Point(224, 8);
			this.hooligansGroupBox.Name = "hooligansGroupBox";
			this.hooligansGroupBox.Size = new System.Drawing.Size(304, 120);
			this.hooligansGroupBox.TabIndex = 2;
			this.hooligansGroupBox.TabStop = false;
			this.hooligansGroupBox.Text = "";
			this.hooligansGroupBox.Enter += new System.EventHandler(this.groupBox1_Enter);
			// 
			// offLabel
			// 
			this.offLabel.Location = new System.Drawing.Point(48, 120);
			this.offLabel.Name = "offLabel";
			this.offLabel.TabIndex = 8;
			this.offLabel.Text = "";
			// 
			// onLabel
			// 
			this.onLabel.Location = new System.Drawing.Point(48, 104);
			this.onLabel.Name = "onLabel";
			this.onLabel.TabIndex = 7;
			this.onLabel.Text = "";
			// 
			// autoLabel
			// 
			this.autoLabel.Location = new System.Drawing.Point(48, 56);
			this.autoLabel.Name = "autoLabel";
			this.autoLabel.TabIndex = 6;
			this.autoLabel.Text = "";
			// 
			// manualLabel
			// 
			this.manualLabel.Location = new System.Drawing.Point(48, 40);
			this.manualLabel.Name = "manualLabel";
			this.manualLabel.TabIndex = 5;
			this.manualLabel.Text = "";
			this.manualLabel.Click += new System.EventHandler(this.label4_Click);
			// 
			// blinkLabel
			// 
			this.blinkLabel.Location = new System.Drawing.Point(48, 24);
			this.blinkLabel.Name = "blinkLabel";
			this.blinkLabel.TabIndex = 4;
			this.blinkLabel.Text = "";
			// 
			// smallButtonSlider
			// 
			this.smallButtonSlider.Location = new System.Drawing.Point(16, 96);
			this.smallButtonSlider.Maximum = 1;
			this.smallButtonSlider.Name = "smallButtonSlider";
			this.smallButtonSlider.Orientation = System.Windows.Forms.Orientation.Vertical;
			this.smallButtonSlider.Size = new System.Drawing.Size(42, 40);
			this.smallButtonSlider.TabIndex = 3;
			this.smallButtonSlider.TickStyle = System.Windows.Forms.TickStyle.None;
			this.smallButtonSlider.Scroll += new System.EventHandler(this.trackBar2_Scroll);
			// 
			// modeSlider
			// 
			this.modeSlider.Location = new System.Drawing.Point(16, 24);
			this.modeSlider.Maximum = 2;
			this.modeSlider.Name = "modeSlider";
			this.modeSlider.Orientation = System.Windows.Forms.Orientation.Vertical;
			this.modeSlider.Size = new System.Drawing.Size(42, 48);
			this.modeSlider.TabIndex = 2;
			this.modeSlider.TickStyle = System.Windows.Forms.TickStyle.None;
			this.modeSlider.Scroll += new System.EventHandler(this.trackBar1_Scroll);
			// 
			// switchButton
			// 
			this.switchButton.Location = new System.Drawing.Point(208, 56);
			this.switchButton.Name = "switchButton";
			this.switchButton.Size = new System.Drawing.Size(88, 23);
			this.switchButton.TabIndex = 1;
			this.switchButton.Text = "!";
			this.switchButton.Click += new System.EventHandler(this.button3_Click);
			// 
			// powerButton
			// 
			this.powerButton.Location = new System.Drawing.Point(216, 24);
			this.powerButton.Name = "powerButton";
			this.powerButton.Size = new System.Drawing.Size(80, 23);
			this.powerButton.TabIndex = 0;
			this.powerButton.Text = "";
			this.powerButton.Click += new System.EventHandler(this.button2_Click);
			// 
			// controlPanelGroupBox
			// 
			this.controlPanelGroupBox.Controls.Add(this.offLabel);
			this.controlPanelGroupBox.Controls.Add(this.onLabel);
			this.controlPanelGroupBox.Controls.Add(this.autoLabel);
			this.controlPanelGroupBox.Controls.Add(this.manualLabel);
			this.controlPanelGroupBox.Controls.Add(this.blinkLabel);
			this.controlPanelGroupBox.Controls.Add(this.smallButtonSlider);
			this.controlPanelGroupBox.Controls.Add(this.modeSlider);
			this.controlPanelGroupBox.Controls.Add(this.switchButton);
			this.controlPanelGroupBox.Controls.Add(this.powerButton);
			this.controlPanelGroupBox.Location = new System.Drawing.Point(224, 144);
			this.controlPanelGroupBox.Name = "controlPanelGroupBox";
			this.controlPanelGroupBox.Size = new System.Drawing.Size(304, 152);
			this.controlPanelGroupBox.TabIndex = 3;
			this.controlPanelGroupBox.TabStop = false;
			this.controlPanelGroupBox.Text = " ";
			// 
			// smallButton
			// 
			this.smallButton.Location = new System.Drawing.Point(200, 128);
			this.smallButton.Name = "smallButton";
			this.smallButton.Size = new System.Drawing.Size(16, 16);
			this.smallButton.TabIndex = 1;
			this.smallButton.Text = "*";
			this.smallButton.Click += new System.EventHandler(this.button1_Click);
			// 
			// blinkTimer
			// 
			this.blinkTimer.Interval = 1000;
			this.blinkTimer.Tick += new System.EventHandler(this.blinkTimer_Tick);
			// 
			// longTimer
			// 
			this.longTimer.Tick += new System.EventHandler(this.longTimer_Tick);
			// 
			// shortTimer
			// 
			this.shortTimer.Tick += new System.EventHandler(this.shortTimer_Tick);
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(568, 309);
			this.Controls.Add(this.controlPanelGroupBox);
			this.Controls.Add(this.hooligansGroupBox);
			this.Controls.Add(this.smallButton);
			this.Controls.Add(this.drawPanel);
			this.Name = "MainForm";
			this.Text = "";
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.Form1_Paint);
			this.hooligansGroupBox.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.smallButtonSlider)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.modeSlider)).EndInit();
			this.controlPanelGroupBox.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion

	
		[STAThread]
		static void Main() 
		{
			Application.Run(new MainForm());
		}

		private void Form1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
		}

		//      		
		private void button4_Click(object sender, System.EventArgs e)
		{
			a0.brokenred();
		}

		//      
		private void button5_Click(object sender, System.EventArgs e)
		{
			a0.brokenyellow();
		}

		//      
		private void button6_Click(object sender, System.EventArgs e)
		{
			a0.brokengreen();
		}

		//      
		private void button7_Click(object sender, System.EventArgs e)
		{
			a0.fixedred();
		}

		//      
		private void button8_Click(object sender, System.EventArgs e)
		{
			a0.fixedyellow();
		}

		//      
		private void button9_Click(object sender, System.EventArgs e)
		{
			a0.fixedgreen();
		}

		//      
		private void trackBar1_Scroll(object sender, System.EventArgs e)
		{
			switch ( ((TrackBar)sender).Value )
			{
				case 0 :
					a0.auto_mode();
					break;
				case 1 :
					a0.manual_mode();
					break;
				case 2 :
					a0.blinking_mode();
					break;
			}
		}

		//    
		private void button2_Click(object sender, System.EventArgs e)
		{
			a0.power_button();
			if (a0.on)
				powerButton.BackColor = Color.Green;
			else
				powerButton.BackColor = Color.Red;
		}

		//    
		private void button3_Click(object sender, System.EventArgs e)
		{
			a0.switch_lights();
		}

		//    / 
		private void trackBar2_Scroll(object sender, System.EventArgs e)
		{
			a0.switch_button();
		}

		//   
		private void button1_Click(object sender, System.EventArgs e)
		{
			a0.buttonpressed();
		}

		//   
		private void longTimer_Tick(object sender, System.EventArgs e)
		{
			a0.long_time_elapsed();
		}

		//   
		private void shortTimer_Tick(object sender, System.EventArgs e)
		{
			a0.short_time_elapsed();
		}

		//    
		private void blinkTimer_Tick(object sender, System.EventArgs e)
		{
			_showLight = !_showLight;
			Redraw( currState );
		}

		private void label4_Click(object sender, System.EventArgs e)
		{
		
		}

		private void groupBox1_Enter(object sender, System.EventArgs e)
		{
		
		}
	}

	//  
	public class Logger
	{
		public static void log_input(int n, string s, bool res)
		{
			Trace.WriteLine(String.Format("I x{0} {1} {2}", n, s, res ? "YES" : "NO" ), "State entry");						
		}
		public static void log_output(int n, string s)
		{
			Trace.WriteLine(String.Format("O z{0} {1}", n, s), "State entry");						
		}
		public static void log_string( string toLog )
		{
			Trace.WriteLine( toLog );
		}
	}

	//   
	public class SupplyFuncs
	{
		private Automata automata;
		public SupplyFuncs( Automata a )
		{
			automata = a;
		}

		public static bool x1(Automata automata)
		{
			bool res = ((Automata0)automata).cur_mode == Automata0.modes.AUTO;
				Logger.log_input(1, " ", res);
			return res;
		}
		public static bool x2(Automata automata)
		{
			bool res = ((Automata0)automata).cur_mode == Automata0.modes.MANUAL;
				Logger.log_input(2, " ", res);
			return res;
		}
		public static bool x3(Automata automata)
		{
			bool res = ((Automata0)automata).button_on;
				Logger.log_input(3, " ", res);
			return res;
		}
		public static bool x4(Automata automata)
		{
			bool res = ((Automata0)automata).broken_red;
				Logger.log_input(4, " ", res);
			return res;
		}
		public static bool x5(Automata automata)
		{
			bool res = ((Automata0)automata).broken_yellow;
				Logger.log_input(5, " ", res);
			return res;
		}

		public static bool x6(Automata automata)
		{
			bool res = ((Automata0)automata).broken_green;
				Logger.log_input(6, " ", res);
			return res;
		}
		public static bool x7(Automata automata)
		{
			bool res = ((Automata0)automata).broken;
				Logger.log_input(7, " ", res);
			return res;
		}
		public static bool x8(Automata automata)
		{
			bool res = ((Automata0)automata).button_pressed;
				Logger.log_input(8, "  ''", res);
			return res;
		}

		public static bool x9(Automata automata)
		{
			bool res = ((Automata0)automata).cur_mode == Automata0.modes.BLINKING;
				Logger.log_input(9, " ", res);
			return res;
		}
	}

	// ,   
	public class Cond1 : Condition
	{
		public Cond1( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return ((e == 5) || SupplyFuncs.x7(this._switch)) && !SupplyFuncs.x4(this._switch) && !SupplyFuncs.x5(this._switch) && !SupplyFuncs.x6(this._switch) && !SupplyFuncs.x9(this._switch);
		}
	}

	public class Cond2 : Condition
	{
		public Cond2( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return (e == 5) && ( SupplyFuncs.x9(this._switch) || SupplyFuncs.x4(this._switch) || SupplyFuncs.x6(this._switch));
		}
	}
	public class Cond3 : Condition
	{
		public Cond3( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return (e == 6) || SupplyFuncs.x5(this._switch);
		}
	}
	public class Cond4 : Condition
	{
		public Cond4( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			bool res = ( (e == 3) || (e == 9) || (SupplyFuncs.x7(this._switch) && !SupplyFuncs.x9(this._switch))  ) && !SupplyFuncs.x4(this._switch) && !SupplyFuncs.x6(this._switch);
			return res;
		}
	}
	public class Cond5 : Condition
	{
		public Cond5( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return ( ( e == 7 ) && SupplyFuncs.x1(this._switch) ) || ( ( e == 4 ) && SupplyFuncs.x2(this._switch) ) || ( e == 3 );
		}
	}
	public class Cond6 : Condition
	{
		public Cond6( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return e == 8;
		}
	}
	public class Cond7 : Condition
	{
		public Cond7( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return ( e == 7 ) && (SupplyFuncs.x1(this._switch)) && ( !SupplyFuncs.x3(this._switch) || SupplyFuncs.x3(this._switch) && SupplyFuncs.x8(this._switch)) || (e == 4) && SupplyFuncs.x2(this._switch) || (e == 3);
		}
	}
	public class Cond8 : Condition
	{
		public Cond8( Automata sw ) : base( sw )
		{

		}

		public override bool Test( int e )
		{
			return ( e == 7 ) && SupplyFuncs.x1(this._switch) && SupplyFuncs.x3(this._switch) && !SupplyFuncs.x8(this._switch);
		}
	}
	public class Cond9 : Condition
	{
		public Cond9( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return (( e == 1 ) && SupplyFuncs.x1(this._switch)) || ( (e == 4) && SupplyFuncs.x2(this._switch) ) || (e == 3);
		}
	}
	public class Cond10 : Condition
	{
		public Cond10( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return SupplyFuncs.x4(this._switch) || SupplyFuncs.x6(this._switch);
		}
	}
	public class Cond11 : Condition
	{
		public Cond11( Automata sw ) : base( sw )
		{
		}

		public override bool Test( int e )
		{
			return e == 2;
		}
	}

	// ,   
	public class ActionZ1 : Action
	{
		private Automata _a;
		public ActionZ1( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).broken = true;
				Logger.log_output(1, " ");

		}
	}

	public class ActionZ2 : Action
	{
		private Automata _a;
		public ActionZ2( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((MainForm)((Automata0)_a).Args).longTimer.Interval = 10000;
			((MainForm)((Automata0)_a).Args).longTimer.Start();
			Logger.log_output(2, " '' ");
		}
	}

	public class ActionZ3 : Action
	{
		private Automata _a;
		public ActionZ3( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((MainForm)((Automata0)_a).Args).shortTimer.Interval = 6000;
			((MainForm)((Automata0)_a).Args).shortTimer.Start();
			Logger.log_output(3, " '' ");
		}
	}
	public class ActionZ4 : Action
	{
		private Automata _a;
		public ActionZ4( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.GREEN);
				Logger.log_output(4, " ");
		}
	}
	public class ActionZ5 : Action
	{
		private Automata _a;
		public ActionZ5( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.BLINKGREEN);
			Logger.log_output(5, "  ");
		}
	}
	public class ActionZ6 : Action
	{
		private Automata _a;
		public ActionZ6( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.YELLOW);
//			if(LOGGING)
				Logger.log_output(6, " ");
		}
	}
	public class ActionZ7 : Action
	{
		private Automata _a;
		public ActionZ7( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.RED);
				Logger.log_output(7, " ");
		}
	}
	public class ActionZ8 : Action
	{
		private Automata _a;
		public ActionZ8( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.REDYELLOW);
				Logger.log_output(8, "   ");
		}
	}
	public class ActionZ9 : Action
	{
		private Automata _a;
		public ActionZ9( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.BLINKYELLOW);
				Logger.log_output(9, "  ");
		}
	}
	public class ActionZ10 : Action
	{
		private Automata _a;
		public ActionZ10( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).light(Automata0.states.OFF);
				Logger.log_output(10, " ");
		}
	}
	public class ActionZ11 : Action
	{
		private Automata _a;
		public ActionZ11( Automata a ) : base( a )
		{
			_a = a;
		}

		public override void Do()
		{
			((Automata0)_a).button_pressed = false;
				Logger.log_output(11, "  ");
		}
	}

	//  
	public class State0 : State
	{
		private string _id;
		private Automata0 _a;
		public State0( Automata0 a, string id ) : base(a)
		{
			_id = id;
			_a = a;
		}

		public override void Log()
		{
			Logger.log_string( "   '"+_id+"'" );
		}
	}

	//  
	public class Automata0 : Automata
	{

		public State s0, s1, s2, s3, s4, s5, s6, s7, s8;

		public Automata0( State initialState, object o ) : base( initialState, o )
		{
			on = false;

			LOGGING = true;
			cur_mode = modes.AUTO;
			button_on = false;
			broken_red = false;
			broken_yellow = false;
			broken_green = false;
			broken = false;
			button_pressed = false;

			//    State0
			s0 = new State0( this, "0. " );
			s1 = new State0( this, "1. " );
			s2 = new State0( this, "2.  "  );
			s3 = new State0( this, "3. "  );
			s4 = new State0( this, "4. "  );
			s5 = new State0( this, "5.  "  );
			s6 = new State0( this, "6.   "  );
			s7 = new State0( this, "7.  "  );

			// 
			State = s0;

			//     
			s0.AddAction( new ActionZ10( this ) );

			s1.AddAction( new ActionZ2( this ) );
			s1.AddAction( new ActionZ4( this ) );

			s2.AddAction( new ActionZ3( this ) );
			s2.AddAction( new ActionZ5( this ) );

			s3.AddAction( new ActionZ3( this ) );
			s3.AddAction( new ActionZ6( this ) );

			s4.AddAction( new ActionZ3( this ) );
			s4.AddAction( new ActionZ7( this ) );
			s4.AddAction( new ActionZ11( this ) );

			s6.AddAction( new ActionZ3( this ) );
			s6.AddAction( new ActionZ8( this ) );

			s7.AddAction( new ActionZ9( this ) );

			//  
			Transitions.Add(s0, s1, new Cond1( this ), (Action) null);
			Transitions.Add(s1, s2, new Cond5( this ), (Action) null);
			Transitions.Add(s2, s3, new Cond6( this ), (Action) null);
			Transitions.Add(s3, s4, new Cond6( this ), (Action) null);
			Transitions.Add(s4, s6, new Cond7( this ), (Action) null);
			Transitions.Add(s4, s5, new Cond8( this ), (Action) null);
			Transitions.Add(s5, s6, new Cond9( this ), (Action) null);
			Transitions.Add(s6, s1, new Cond6( this ), (Action) null);
			Transitions.Add(s1, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s1, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s2, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s2, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s3, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s3, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s4, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s4, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s5, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s5, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s6, s7, new Cond11( this ), (Action) null);
			Transitions.Add(s6, s7, new Cond10( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s7, s1, new Cond4( this ), (Action) null);
			Transitions.Add(s0, s7, new Cond2( this ), (Action) null);

			Transitions.Add(s1, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s2, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s3, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s4, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s5, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s6, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
			Transitions.Add(s7, s0, new Cond3( this ), (Action) new ActionZ1( this ) );
		}

		//  

		//    (e3)
		public void auto_mode()		
		{
			Execute(3);
			cur_mode = modes.AUTO;
		}

		//    (e9)
		public void manual_mode()
		{
			Execute(9);
			cur_mode = modes.MANUAL;
		}

		//    (e2)
		public void blinking_mode()
		{
			cur_mode = modes.BLINKING;
			Execute(2);
		}

		
		public void switch_button()		
		{
			button_on = !button_on;
		}

		// /  (e5, e6)
		public void power_button()
		{
			if(on)
			{
				on = false;
				Execute(6);
			}
			else
			{
				on = true;
				Execute(5);
			}
		}

		//   (e4)
		public void switch_lights()		
		{
			Execute(4);
		}

		// C 
		public void brokenred()
		{
			broken_red = true;
			Execute(0);
		}

		//  
		public void brokengreen()			
		{
			broken_green = true;
			Execute(0);
		}

		//  
		public void brokenyellow()		
		{
			broken_yellow = true;
			Execute(0);
		}

		//  
		public void fixedred()
		{
			broken_red = false;
			Execute(0);
		}

		//  
		public void fixedgreen()		
		{
			broken_green = false;
			Execute(0);
		}

		//  
		public void fixedyellow()			
		{
			broken_yellow = false;
			Execute(0);
		}
	
		//    (e1)
		public void buttonpressed()
		{
			button_pressed = true;
			Execute(1);
		}

		//     (e7)
		public void long_time_elapsed()	
		{
			Execute(7);
		}

		//    
		public void short_time_elapsed()	
		{
			Execute(8);
		}

		public void light(states what)
		{
			((MainForm)this.Args).ChangeLight(what);
		}

		public modes cur_mode;
		public bool	blink_mode;
		public bool	button_on;
		public bool	broken_red;
		public bool	broken_yellow;
		public bool	broken_green;
		public bool	broken;
		public bool	button_pressed;
		public int		y0;
		//  
		public bool		LOGGING;
		// ofstream	logfile;
		public bool		on;
		//   cur_mode
		public enum modes {AUTO, MANUAL, BLINKING};
		//  
		public enum timers {LTIMER=1, STIMER=2};

		public enum states 
		{
			RED,
			YELLOW, 
			GREEN, 
			REDYELLOW, 
			BLINKGREEN, 
			BLINKYELLOW,
			OFF
		};


	}
}
